<?php
/**
 * Get order list.
 * 
 * @param  string    $mode 
 * @param  mix       $value 
 * @param  string    $orderBy 
 * @param  object    $pager 
 * @access public
 * @return array
 */
public function getList($mode, $value, $orderBy = 'id_desc', $pager = null)
{
    $days = $this->config->shop->confirmLimit;

    if($days)
    {
        $deliveryDate = date('Y-m-d H:i:s', time() - 24 * 60 * 60 * $days);
        $this->dao->update(TABLE_ORDER)
            ->set('deliveryStatus')->eq('confirmed')
            ->where('deliveryStatus')->eq('send')
            ->andWhere('deliveriedDate')->le($deliveryDate)
            ->exec();
    }

    $orders = $this->dao->select('t1.*')->from(TABLE_ORDER)->alias('t1')
        ->leftJoin(TABLE_ADDRESS)->alias('t2')->on('t1.account = t2.account')
        ->beginIf($mode == 'account')->where('t1.account')->eq($value)->fi()
        ->beginIf($mode == 'status')->where('t1.status')->eq($value)->fi()
        ->beginIf(!commonModel::isAvailable('score'))->andWhere('t1.type')->ne('score')->fi()
        ->beginIf(!commonModel::isAvailable('shop'))->andWhere('t1.type')->ne('shop')->fi()
        ->beginIf($this->get->contact)->andwhere('t2.contact')->eq($this->get->contact)->fi()
        ->beginIf($this->get->phone)->andwhere('t2.phone')->eq($this->get->phone)->fi()
        ->orderBy($orderBy)
        ->page($pager)
        ->fetchAll('id');

    $products = $this->dao->select('*')->from(TABLE_ORDER_PRODUCT)
        ->where('orderID')->in(array_keys($orders))
        ->beginIf($this->get->product)->andWhere('productID')->eq($this->get->product)->fi()
        ->fetchGroup('orderID');

    foreach($orders as $key => $order)
    {
        if($this->get->product and !isset($products[$order->id]))
        {
            unset($orders[$key]);
            continue;
        }
        $order->products = isset($products[$order->id]) ? $products[$order->id] : array();
    }

    return $orders;
}

/**
 * Get product infomation posted to buy.
 * 
 * @param  string $product 
 * @param  int    $count 
 * @access public
 * @return void
 */
public function getPostedProducts($product, $count = 0)
{
    $productIdList  = $this->post->product;

    /* Get products(use groupBy to distinct products).  */
    $products = $this->dao->select('t1.*, t2.category')->from(TABLE_PRODUCT)->alias('t1')
        ->leftJoin(TABLE_RELATION)->alias('t2')->on('t1.id = t2.id')
        ->where('t1.id')->in($productIdList)
        ->andWhere('t2.type')->eq('product')
        ->beginIF(RUN_MODE == 'front')->andWhere('t1.status')->eq('normal')->fi()
        ->fetchAll('id');

    if(empty($products)) return array();

    /* Get categories for these products. */
    $categories = $this->dao->select('t2.id, t2.name, t2.alias, t1.id AS product')
        ->from(TABLE_RELATION)->alias('t1')
        ->leftJoin(TABLE_CATEGORY)->alias('t2')->on('t1.category = t2.id')
        ->where('t2.type')->eq('product')
        ->andWhere('t1.id')->in(array_keys($products))
        ->fetchGroup('product', 'id');

    /* Assign categories to it's product. */
    foreach($products as $product) $product->categories = !empty($categories[$product->id]) ? $categories[$product->id] : array();

    /* Get images for these products. */
    $images = $this->loadModel('file')->getByObject('product', array_keys($products), $isImage = true);

    /* Assign images to it's product. */
    foreach($products as $product)
    {
        if(empty($images[$product->id])) continue;
        $product->image = new stdclass();
        if(isset($images[$product->id]))  $product->image->list = $images[$product->id];
        if(!empty($product->image->list)) $product->image->primary = $product->image->list[0];

    }

    $goodsList = array();
    foreach($this->post->product as $id => $product)
    {
        $goods = new stdclass();

        $product = zget($products, $product);

        $goods->product    = $product->id;
        $goods->name       = $product->name;
        $goods->image      = isset($product->image) ? $product->image : '';
        $goods->alias      = $product->alias;
        $goods->category   = $product->category;
        $goods->categories = $product->categories;
        $goods->count      = $this->post->count[$id];
        $goods->amount     = $product->amount;
        $goods->extra      = (object) $this->post->extra[$id];
        $goods->attributes = $this->loadModel('attribute')->getPairs(array_keys($product->categories));

        $goods->price = $this->loadModel('product')->getPriceByAttr($product, $goods->extra);
        $goods->promotion = 0;

        $goodsList[$id] = $goods;
    }

    return $goodsList;
}

/**
 * Create an order.
 * 
 * @access public
 * @return void
 */
public function create()
{
    $order = fixer::input('post')
        ->add('account', $this->app->user->account)
        ->add('createdDate', helper::now())
        ->add('payStatus', 'not_paid')
        ->add('status', 'normal')
        ->add('deliveryStatus', 'not_send')
        ->add('type', 'shop')
        ->get();

    $address = $this->dao->select('*')->from(TABLE_ADDRESS)->where('id')->eq($this->post->deliveryAddress)->andWhere('account')->eq($this->app->user->account)->fetch();
    $order->address = helper::jsonEncode($address);

    if($this->post->createAddress)
    {
        $address = $this->createAddress();
        if(!$address) return array('result' => 'fail', 'message' => dao::getError());
        $order->address = helper::jsonEncode($address);
    }

    $this->dao->insert(TABLE_ORDER)
        ->data($order, 'createAddress,deliveryAddress,zipcode,phone,contact,price,count,product,extra')
        ->autocheck()
        ->batchCheck($this->config->order->require->create, 'notempty')
        ->exec();
    if(dao::isError()) return array('result' => 'fail', 'message' => dao::getError());

    $orderID = $this->dao->lastInsertID();

    $goods = new stdclass();
    $goods->orderID = $orderID;

    if(!$this->post->product)
    {
        $this->clearOrder($orderID);
        return array('result' => 'fail', 'message' => $this->lang->order->noProducts);
    }
    
    /* Save products of the order and compute order amount. */
    $amount = 0;
    foreach($this->post->product as $id => $product)
    {
        $product = $this->dao->select('*')->from(TABLE_PRODUCT)->where('id')->eq($product)->fetch();
        if(empty($product)) continue;

        $goods->productID   = $product->id; 
        $goods->productName = $product->name; 
        $goods->count       = $this->post->count[$id];
        $goods->extra       = json_encode($this->post->extra[$id]);

        if(isset($this->config->product->stock) && $this->config->product->stock)
        {
            if($product->amount < $goods->count)
            {
                $this->clearOrder($orderID);
                return array('result' => 'fail', 'message' => sprintf($this->lang->order->lowStocks, $goods->productName));
            }
        }

        $goods->price = $this->loadModel('product')->getPriceByAttr($product, json_decode($goods->extra));

        $amount += $goods->price * $goods->count;

        $this->dao->insert(TABLE_ORDER_PRODUCT)->data($goods)->autoCheck()->exec();
    }

    /* Check valid products count. */
    $productCout = $this->dao->select("count(*) as count")->from(TABLE_ORDER_PRODUCT)->where('orderID')->eq($orderID)->fetch('count');
    if(!$productCout)  return array('result' => 'fail', 'message' => $this->lang->order->noProducts);

    $this->dao->update(TABLE_ORDER)->set('amount')->eq($amount)->where('id')->eq($orderID)->exec();
    $this->dao->delete()->from(TABLE_CART)->where('account')->eq($this->app->user->account)->andWhere('product')->in($this->post->product)->exec();
    if(!dao::isError()) return $orderID;
}

/**
 * Process an order.
 * 
 * @param  object $order
 * @access public
 * @return bool
 */
public function processOrder($order)
{
    if($order->payStatus == 'paid') return true;

    $this->dao->update(TABLE_ORDER)
        ->set('sn')->eq($order->sn)
        ->set('payStatus')->eq('paid')
        ->set('paidDate')->eq(helper::now())
        ->where('id')->eq($order->id)->exec();

    if(dao::isError()) return false;

    $accounts  = $this->dao->select('mobile')->from(TABLE_USER)->where('account')->in($this->config->shop->sendto)->fetchAll();
    $mobiles   = array();
    $delimiter = isset($this->app->config->sms->delimiter) ? $this->app->config->sms->delimiter : ',';
    foreach($accounts as $account)
    {   
        if($account->mobile) $mobiles[] = $account->mobile;
    }

    $content = sprintf($this->lang->order->toAdminContent, $this->config->site->name, $this->server->http_host, $order->account, $order->createdDate);

    //$this->loadModel('sms')->send(join($mobiles, $delimiter), $content);
    $this->loadModel('mail')->send($this->config->shop->sendto, $this->lang->order->sendSubject, $content, true);

    return true;
}

/**
 * Delivery products of an order.
 * 
 * @param  int    $orderID 
 * @access public
 * @return void
 */
public function delivery($orderID)
{
    $order = $this->getByID($orderID);

    $delivery = fixer::input('post')
        ->add('deliveriedBy', $this->app->user->account)
        ->add('deliveryStatus', 'send')
        ->get();

    $this->dao->update(TABLE_ORDER)->data($delivery)->where('id')->eq($orderID)->exec();

    if(dao::isError()) return array('result' => 'fail', 'message' => dao::getError());

    if(isset($this->config->product->stock) and $this->config->product->stock)
    {
        $goodsList = $this->dao->select('*')->from(TABLE_ORDER_PRODUCT)->where('orderID')->eq($orderID)->fetchAll();
        foreach($goodsList as $goods)
        {
            $product = $this->dao->select('*')->from(TABLE_PRODUCT)->where('id')->eq($goods->productID)->fetch();
            if($product->amount < $goods->count)
            {
                return array('result' => 'fail', 'message' => strip_tags(sprintf($this->lang->order->lowStocks, $goods->productName)));
            }
        }
    }

    if($order->payment == 'COD' and isset($this->config->product->stock) and $this->config->product->stock) $this->fixStocks($orderID);

    $address     = json_decode($order->address);
    $expressList = $this->loadModel('tree')->getOptionMenu('express');

    $mobile  = $address->mobile;
    $content = sprintf($this->lang->order->toCustomerContent, $order->account, $this->config->site->name, $this->server->http_host, $expressList[$delivery->express], $delivery->waybill);
    $this->loadModel('sms')->send($mobile, $content);

    return array('result' => 'success', 'message' => $this->lang->saveSuccess, 'locate' => inlink('admin'));
}

/**
 * Print actions for order.
 * 
 * @param  int    $order 
 * @param  int    $btnLink 
 * @access public
 * @return void
 */
public function printActions($order, $btnLink = false)
{
    parent::printActions($order, $btnLink);

    if(RUN_MODE == 'front' and $order->status == 'normal')
    {
        if($btnLink)
        {
            /* message link. */
            echo html::a(helper::createLink('message', 'comment', "objectType=order&objectID=$order->id"), $this->lang->order->message, "data-toggle='modal' class='btn warning'");
        }
        else
        {
            /* message link. */
            echo html::a(helper::createLink('message', 'comment', "objectType=order&objectID=$order->id"), $this->lang->order->message, "data-toggle='modal'");
        }
    }

    if(RUN_MODE == 'admin' and $order->status == 'normal')
    {
        commonModel::printLink('order', 'editprice', "orderID={$order->id}", $this->lang->order->editPrice, "data-toggle='modal' ");

        $trackClass = $order->deliveryStatus == 'send' ? "" : "class='disabled'";
        commonModel::printLink('order', 'track', "orderID={$order->id}", $this->lang->order->track, "{$trackClass} data-toggle='modal' ");

        if(!$btnLink)
        {
            /* message link. */
            echo html::a(helper::createLink('order', 'message', "objectID=$order->id"), $this->lang->order->message, "data-toggle='modal'");
        }
    }
}

public function editPrice($orderID)
{
    $this->dao->update(TABLE_ORDER)->set('amount')->eq($this->post->amount)->where('id')->eq($orderID)->exec();
    return !dao::isError();
}
